@@ -1,8 +1,9 @@
 /*
- * dselect - Debian GNU/Linux package maintenance user interface
+ * dselect - Debian package maintenance user interface
  * bcommands.cc - base list keyboard commands display
  *
- * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright (C) 2000,2001 Wichert Akkerman <wakkerma@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -18,13 +19,14 @@
  * License along with this; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-
+extern "C" {
+#include <config.h>
+}
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
 
 extern "C" {
-#include <config.h>
 #include <dpkg.h>
 #include <dpkg-db.h>
 }
@@ -92,13 +94,32 @@ void baselist::kd_searchagain() {
   dosearch();
 }
 
+int baselist::checksearch(char* str) {
+  return 1;
+}
+
+int baselist::matchsearch(int index) {
+  int lendiff, searchlen, i;
+  const char* thisname;
+
+  thisname=itemname(index);
+  if (!thisname) return 0;	/* Skip things without a name (seperators) */
+
+  searchlen=strlen(searchstring);
+  lendiff= strlen(thisname) - searchlen;
+  for (i=0; i<=lendiff; i++)
+    if (!strncasecmp(thisname + i, searchstring, searchlen))
+      return 1;
+
+  return 0;
+}
+
 void baselist::kd_search() {
-  char newsearchstring[50];
+  char newsearchstring[128];
   strcpy(newsearchstring,searchstring);
   werase(querywin);
   mvwaddstr(querywin,0,0, _("Search for ? "));
   echo(); /* fixme: ncurses documentation or implementation */
-  /* fixme: make / RET do `search again' and / DEL to abort */
   if (wgetnstr(querywin,newsearchstring,sizeof(newsearchstring)-1) == ERR)
     searchstring[0]= 0;
   raise(SIGWINCH); /* fixme: ncurses and xterm arrow keys */
@@ -108,23 +129,43 @@ void baselist::kd_search() {
   else if (thisstate_height) redrawthisstate();
   else { touchwin(listpad); refreshlist(); }
   if (newsearchstring[0]) {
+    if (!checksearch(newsearchstring)) {
+      beep();
+      return;
+    }
     strcpy(searchstring,newsearchstring);
     dosearch();
   } else
     kd_searchagain();
 }
 
+void baselist::displayerror(const char* str) {
+  const char* pr = _("Error: ");
+  int prlen=strlen(pr);
+
+  beep();
+  werase(querywin);
+  mvwaddstr(querywin,0,0,pr);
+  mvwaddstr(querywin,0,prlen,str);
+  wgetch(querywin);
+  if (whatinfo_height) { touchwin(whatinfowin); refreshinfo(); }
+  else if (info_height) { touchwin(infopad); refreshinfo(); }
+  else if (thisstate_height) redrawthisstate();
+}
+
+
 void baselist::displayhelp(const struct helpmenuentry *helpmenu, int key) {
   const struct helpmenuentry *hme;
   int maxx, maxy, i, y, x, nextkey;
   
   getmaxyx(stdscr,maxy,maxx);
+  wbkgdset(stdscr, ' ' | helpscreen_attr);
   clearok(stdscr,TRUE);
   for (;;) {
     werase(stdscr);
     for (hme= helpmenu; hme->key && hme->key != key; hme++);
     if (hme->key) {
-      attrset(list_attr);
+      attrset(helpscreen_attr);
       mvaddstr(1,0, gettext(hme->msg->text));
       attrset(title_attr);
       mvaddstr(0,0, _("Help: "));
@@ -133,8 +174,7 @@ void baselist::displayhelp(const struct helpmenuentry *helpmenu, int key) {
       while (++x<maxx) addch(' ');
       attrset(thisstate_attr);
       mvaddstr(maxy-1,0,
-_("? = help menu    Space = exit help    . = next help    or a help page key ")
-               );
+_("Press ? for help menu, . for next topic, <space> to exit help."));
       getyx(stdscr,y,x);
       while (++x<maxx) addch(' ');
       move(maxy,maxx);
@@ -149,16 +189,16 @@ _("? = help menu    Space = exit help    . = next help    or a help page key ")
         mvaddstr(i+3,6, gettext(hme->msg->title));
       }
       mvaddstr(i+4,1,
-               _("Press a key from the list above, Space to exit help,\n"
+               _("Press a key from the list above, <space> or `q' to exit help,\n"
                "  or `.' (full stop) to read each help page in turn. "));
       nextkey= helpmenu[0].key;
     }
     refresh();
     key= getch();
     if (key == EOF) ohshite(_("error reading keyboard in help"));
-    if (key == ' ') {
+    if (key == ' ' || key == 'q') {
       break;
-    } else if (key == '?') {
+    } else if (key == '?' || key == 'h') {
       key= 0;
     } else if (key == '.') {
       key= nextkey;
