@@ -40,6 +40,7 @@
 #include <dpkg-db.h>
 
 char *statusfile=NULL, *availablefile=NULL;
+char *triggersdir, *triggersfilefile, *triggersnewfilefile;
 
 static enum modstatdb_rw cstatus=-1, cflags=0;
 static char *importanttmpfile=NULL;
@@ -122,10 +123,16 @@ static void createimptmp(void) {
   onerr_abort--;
 }
 
-const struct fni { const char *suffix; char **store; } fnis[]= {
+static const struct fni {
+  const char *suffix;
+  char **store;
+} fnis[] = {
   {   STATUSFILE,                 &statusfile         },
   {   AVAILFILE,                  &availablefile      },
   {   UPDATESDIR IMPORTANTTMP,    &importanttmpfile   },
+  {   TRIGGERSDIR,                &triggersdir        },
+  {   TRIGGERSDIR "/File",        &triggersfilefile   },
+  {   TRIGGERSDIR "/File.new",    &triggersnewfilefile},
   {   NULL, NULL                                      }
 };
 
@@ -189,10 +196,13 @@ enum modstatdb_rw modstatdb_init(const char *adir, enum modstatdb_rw readwritere
     uvb.buf= m_malloc(uvb.size);
   }
 
+  trig_fixup_awaiters(cstatus);
+  trig_incorporate(cstatus, admindir);
+
   return cstatus;
 }
 
-static void checkpoint(void) {
+void modstatdb_checkpoint(void) {
   int i;
 
   assert(cstatus >= msdbrw_write);
@@ -211,7 +221,7 @@ void modstatdb_shutdown(void) {
   const struct fni *fnip;
   switch (cstatus) {
   case msdbrw_write:
-    checkpoint();
+    modstatdb_checkpoint();
     writedb(availablefile,1,0);
     /* tidy up a bit, but don't worry too much about failure */
     fclose(importanttmp);
@@ -219,7 +229,7 @@ void modstatdb_shutdown(void) {
     varbuffree(&uvb);
     /* fall through */
   case msdbrw_needsuperuserlockonly:
-    unlockdatabase(admindir);
+    unlockdatabase();
   default:
     break;
   }
@@ -231,33 +241,14 @@ void modstatdb_shutdown(void) {
   free(updatefnbuf);
 }
 
-struct pipef *status_pipes= NULL;
-
-void modstatdb_note(struct pkginfo *pkg) {
+static void
+modstatdb_note_core(struct pkginfo *pkg)
+{
   assert(cstatus >= msdbrw_write);
 
-  onerr_abort++;
-
-  if (status_pipes) {
-    static struct varbuf *status= NULL;
-    struct pipef *pipef= status_pipes;
-    int r;
-    if (status == NULL) {
-      status = nfmalloc(sizeof(struct varbuf));
-      varbufinit(status);
-    } else
-      varbufreset(status);
-    r= varbufprintf(status, "status: %s: %s\n", pkg->name, statusinfos[pkg->status].name);
-    while (pipef) {
-      write(pipef->fd, status->buf, r);
-      pipef= pipef->next;
-    }
-  }
-  log_message("status %s %s %s", statusinfos[pkg->status].name, pkg->name,
-	      versiondescribe(&pkg->installed.version, vdew_nonambig));
-
   varbufreset(&uvb);
   varbufrecord(&uvb, pkg, &pkg->installed);
+
   if (fwrite(uvb.buf, 1, uvb.used, importanttmp) != uvb.used)
     ohshite(_("unable to write updated status of `%.250s'"), pkg->name);
   if (fflush(importanttmp))
@@ -271,56 +262,78 @@ void modstatdb_note(struct pkginfo *pkg) {
   sprintf(updatefnrest, IMPORTANTFMT, nextupdate);
   if (rename(importanttmpfile, updatefnbuf))
     ohshite(_("unable to install updated status of `%.250s'"), pkg->name);
-  assert(strlen(updatefnrest)<=IMPORTANTMAXLEN); /* or we've made a real mess */
 
-  nextupdate++;  
+  /* Have we made a real mess? */
+  assert(strlen(updatefnrest) <= IMPORTANTMAXLEN);
+
+  nextupdate++;
 
   if (nextupdate > MAXUPDATES) {
-    checkpoint();
-    nextupdate= 0;
+    modstatdb_checkpoint();
+    nextupdate = 0;
   }
 
   createimptmp();
+}
+
+/* Note: If anyone wants to set some triggers-pending, they must also
+ * set status appropriately, or we will undo it. That is, it is legal
+ * to call this when pkg->status and pkg->trigpend_head disagree and
+ * in that case pkg->status takes precedence and pkg->trigpend_head
+ * will be adjusted.
+ */
+void modstatdb_note(struct pkginfo *pkg) {
+  struct trigaw *ta;
+
+  onerr_abort++;
+
+  if (pkg->status != stat_triggerspending &&
+      pkg->status != stat_triggersawaited)
+    pkg->trigpend_head = NULL;
+
+  if (pkg->status <= stat_configfiles) {
+    for (ta = pkg->trigaw.head; ta; ta = ta->sameaw.next)
+      ta->aw = NULL;
+    pkg->trigaw.head = pkg->trigaw.tail = NULL;
+  }
+
+  log_message("status %s %s %s", statusinfos[pkg->status].name, pkg->name,
+	      versiondescribe(&pkg->installed.version, vdew_nonambig));
+  statusfd_send("status: %s: %s", pkg->name, statusinfos[pkg->status].name);
+
+  if (cstatus >= msdbrw_write)
+    modstatdb_note_core(pkg);
+
+  if (!pkg->trigpend_head && pkg->othertrigaw_head) {
+    /* Automatically remove us from other packages' Triggers-Awaited.
+     * We do this last because we want to maximise our chances of
+     * successfully recording the status of the package we were
+     * pointed at by our caller, although there is some risk of
+     * leaving us in a slightly odd situation which is cleared up
+     * by the trigger handling logic in deppossi_ok_found.
+     */
+    trig_clear_awaiters(pkg);
+  }
 
   onerr_abort--;
 }
 
-const char *log_file= NULL;
-
-void log_message(const char *fmt, ...) {
-  static struct varbuf *log= NULL;
-  static FILE *logfd= NULL;
-  char time_str[20];
-  time_t now;
-  va_list al;
-
-  if (!log_file)
-    return;
-
-  if (!logfd) {
-    logfd= fopen(log_file, "a");
-    if (!logfd) {
-      fprintf(stderr, _("couldn't open log `%s': %s\n"), log_file,
-	      strerror(errno));
-      log_file= NULL;
-      return;
-    }
-    setlinebuf(logfd);
-    setcloexec(fileno(logfd), log_file);
-  }
-
-  if (!log) {
-    log= nfmalloc(sizeof(struct varbuf));
-    varbufinit(log);
-  } else
-    varbufreset(log);
-
-  va_start(al,fmt);
-  varbufvprintf(log, fmt, al);
-  varbufaddc(log, 0);
-  va_end(al);
-
-  time(&now);
-  strftime(time_str, sizeof(time_str), "%Y-%m-%d %H:%M:%S", localtime(&now));
-  fprintf(logfd, "%s %s\n", time_str, log->buf);
+void
+modstatdb_note_ifwrite(struct pkginfo *pkg)
+{
+  if (cstatus >= msdbrw_write)
+    modstatdb_note(pkg);
 }
+
+const char *pkgadminfile(struct pkginfo *pkg, const char *whichfile) {
+  static struct varbuf vb;
+  varbufreset(&vb);
+  varbufaddstr(&vb,admindir);
+  varbufaddstr(&vb,"/" INFODIR);
+  varbufaddstr(&vb,pkg->name);
+  varbufaddc(&vb,'.');
+  varbufaddstr(&vb,whichfile);
+  varbufaddc(&vb,0);
+  return vb.buf;
+}
+
