@@ -36,8 +36,14 @@ void parseerr
   va_list al;
   char buf1[768], buf2[1000], *p, *q;
   if (file && ferror(file)) ohshite(_("failed to read `%s' at line %d"),filename,lno);
-  sprintf(buf1, _("%s, in file `%.255s' near line %d"),
-          warnonly ? _("warning") : _("parse error"), filename, lno);
+
+  if (warnonly)
+    sprintf(buf1, _("warning, in file `%.255s' near line %d"),
+            filename, lno);
+  else
+    sprintf(buf1, _("parse error, in file `%.255s' near line %d"),
+            filename, lno);
+
   if (pigp && pigp->name) {
     sprintf(buf2, _(" package `%.255s'"), pigp->name);
     strcat(buf1,buf2);
@@ -77,11 +83,13 @@ const struct namevalue priorityinfos[]= {  /* Note !  These must be in order ! *
 
 const struct namevalue statusinfos[]= {  /* Note !  These must be in order ! */
   { "not-installed",   stat_notinstalled,    13 },
+  { "config-files",    stat_configfiles,     12 },
+  { "half-installed",  stat_halfinstalled,   14 },
   { "unpacked",        stat_unpacked,        8 },
   { "half-configured", stat_halfconfigured,  15, },
+  { "triggers-awaited", stat_triggersawaited, 16 },
+  { "triggers-pending", stat_triggerspending, 16 },
   { "installed",       stat_installed,       9 },
-  { "half-installed",  stat_halfinstalled,   14 },
-  { "config-files",    stat_configfiles,     12 },
   /* These are additional entries for reading only, in any order ... */
   { "postinst-failed", stat_halfconfigured,  15 }, /* fixme: backwards compat., remove */
   { "removal-failed",  stat_halfinstalled,   14 }, /* fixme: backwards compat., remove */
@@ -119,9 +127,9 @@ const char *illegal_packagename(const char *p, const char **ep) {
     while (isspace(*p)) p++;
     *ep= p; return NULL;
   }
-  snprintf(buf, sizeof(buf),
-          _("character `%c' not allowed - only letters, digits and %s allowed"),
-          c, alsoallowed);
+  snprintf(buf, sizeof(buf), _(
+	   "character `%c' not allowed (only letters, digits and characters `%s')"),
+	   c, alsoallowed);
   return buf;
 }
 
